require 'ticket_assignment'
class TicketsController < ApplicationController
  before_filter :authenticate_user!
  before_filter :load_ticket

  def index
    @tickets = Ticket.all
  end

  def new
    @ticket_types = ticket_types(parent_id: params[:parent_id])
  end

  def create
    @ticket = Ticket.filtered_type_class(params[:type]).new(
      ticket_params.merge(creator: current_user)
    )
    if @ticket.save
      @ticket.create_activity key: "ticket.created", owner: current_user
    end

    @ticket_types = ticket_types(parent_id: @ticket.parent_id)

    respond_to do |format|
      format.js
    end
  end

  def show
    @ticket_types = ticket_types(parent_id: @ticket.id)
    @subtree = @ticket.subtree.includes(:assignments).group_by(&:parent_id)
    @comments = @ticket.comments.group_by(&:id)
  end

  def trigger_event
    old_state = @ticket.state
    if @ticket.trigger_event!(params[:event])
      @activity = @ticket.create_activity key: "ticket.state_change", owner: current_user, parameters: {
        event: params[:event], 
        to: @ticket.state,
        from: old_state
      }
    end
  end

  # only for title/description
  def update
    update_params = params.require(params[:type]).permit(
      :title, :description, 
      :who, :what, :why
    )
    old_title = @ticket.title
    old_description = @ticket.description
    if @ticket.update_attributes(update_params)
      parameters = {}
      if old_title != @ticket.title
        parameters[:title_diff] = generate_diff(old_title, @ticket.title)
        parameters[:title_html_diff] = generate_html_diff(old_title, @ticket.title)
      end
      if old_description != @ticket.description
        parameters[:description_diff] = generate_diff(old_description, @ticket.description)
        parameters[:description_html_diff] = generate_html_diff(old_description, @ticket.description)
      end

      @activity = @ticket.create_activity key: "ticket.update_details", owner: current_user, parameters: parameters
    end
  end

  def assignment
    assignment = TicketAssignment.new @ticket, params[:assignment_user_ids]

    if assignment.assign!
      @activity = @ticket.create_activity key: "ticket.assignment", 
        owner: current_user, 
        parameters: {
          assigned_ids: assignment.assigned_ids, 
          unassigned_ids: assignment.unassigned_ids
        }
    end
  end

  def reorder
    old_parent_id = @ticket.parent_id
    old_position = @ticket.position
    if @ticket.reorder!(
      parent_id: params[:parent_id],
      position: params[:position]
    )
      @activity = @ticket.create_activity key: "ticket.reorder", 
        owner: current_user, 
        parameters: {
          from_parent_id: old_parent_id,
          old_position: old_position,
          parent_id: params[:parent_id], 
          position: params[:position]
        }
    end

    render json: {}
  end

  # FIXME: this is fake right now, need to filter.
  def user_search
    render json: User.all.map { |user|
      {
        name: user.display_name,
        id: user.id,
        value: "#{user.display_name} - #{user.email}"
      }
    }
  end

  private

  def ticket_params
    params.require(params[:type]).permit(
      :title, :description, :parent_id, 
      :assignment_user_ids, :who, :what, :why
    )
  end

  def ticket_types(options = {})
    Ticket::ALLOWED_TYPES.each_with_object({}) do |type, hash|
      hash[type] = Ticket.filtered_type_class(type).new(
        creator: current_user,
        parent_id: options[:parent_id]
      )
    end
  end

  def load_ticket
    @ticket = Ticket.find(params[:id]) if params[:id]
  end

  def generate_diff(old_text, new_text)
    Diffy::Diff.new(
      old_text, new_text, 
      include_diff_info: true,
      include_plus_and_minus_in_html: true,
      context: 2
    )
  end

  def generate_html_diff(old_text, new_text)
    generate_diff(old_text, new_text).to_s(:html)
  end
end
