class Page < ActiveRecord::Base
  include PublicActivity::Common

  include Elasticsearch::Model
  include Elasticsearch::Model::Callbacks

  settings analysis: {
      filter: {
        nGram_filter: Searcher::Filters.ngram
      },
      analyzer: {
        email_tokenizer: {
          tokenizer: "uax_url_email",
          filter: "lowercase"
        },
        nGram_analyzer: Searcher::Analyzers.ngram,
        email_ngram_analyzer: Searcher::Analyzers.email_ngram,
        whitespace_analyzer: Searcher::Analyzers.whitespace
      },
    } do
    mapping dynamic: 'false', index_analyzer: "nGram_analyzer", search_analyzer: "whitespace_analyzer" do
      indexes :id,           :index    => :not_analyzed
      indexes :type,         :as       => :type, :boost => 100
      indexes :ancestry,     :index    => :not_analyzed
      indexes :title,        :analyzer => 'snowball', :boost => 100
      indexes :body,         :analyzer => 'snowball'
      indexes :tag,          :as       => 'tag_list'
      indexes :created_at,   :type => 'date', :include_in_all => false
      indexes :updated_at,   :type => 'date', :include_in_all => false
    end
  end
 
  has_ancestry #touch: true
  acts_as_list scope: [:ancestry]
  acts_as_taggable

  attr_accessor :old_ancestor_ids

  belongs_to :creator_id
  has_many :attachments, as: :attachable, dependent: :destroy

  validates_presence_of :creator
  validates_presence_of :title

  belongs_to :creator, class_name: 'User'
  after_save :touch_ancestry

  def to_param
    [id, title.parameterize].join("-")
  end

  def wiki_dir
    "/wiki/#{id}"
  end

  def update_to_gollum(user)
    commit = { 
      message: 'no message',
      name: user.name,
      email: user.email
    }

    body_page = "#{id}-body"
    title_page = "#{id}-title"

    $wiki.write_page("/" + body_page, :textile, body, commit, wiki_dir) 
    $wiki.write_page("/" + title_page, :textile, title, commit, wiki_dir) 
  end

  def type
    @type || "Page"
  end

  def last_updated
    activities.
      where(key: 'page.update_details').
      order("created_at desc").first ||
      activities.where(key: 'page.created').
      order("created_at desc").first
  end

  def versions
    $wiki.page("#{id}-body", nil, wiki_dir).try(:versions) || []
  end

  def reorder!(options)
    if options[:parent_id]
      self.old_ancestor_ids = self.ancestor_ids
      self.parent_id = options[:parent_id]
      self.save
    end
    self.insert_at options[:position].to_i
    self.save
  end

  def touch_ancestry
    self.old_ancestor_ids ||= []
    Page.where(id: (ancestor_ids + old_ancestor_ids).uniq).update_all(updated_at: Time.now)
  end

  def as_indexed_json(options = {})
    json = self.as_json
    json[:body] = Nokogiri::HTML(self.body || "").text
    json[:tag] = self.tag_list 
    json[:type] = self.class.to_s.underscore
    json
  end

  def save_and_destroy(deleted_by)
    # let's get this page and all child pages and attachments
    # let's save them to deleted_items
    # let's delete them all

    self.children.each do |child|
      child.save_and_destroy(deleted_by)
    end

    self.attachments.each do |attachment|
      attachment.save_and_destroy(deleted_by)
    end

    DeletedItem.save_and_destroy(self, deleted_by)
  end
end
